@@ -31,6 +31,8 @@
 #include "byteArray.h"
 #include "compat.h"
 
+#include "gi.h"
+
 #include <util/log.h>
 #include <util/glib.h>
 #include <util/error.h>
@@ -666,6 +668,12 @@ gjs_context_constructor (GType                  type,
         js_context->profiler = gjs_profiler_new(js_context->runtime);
     }
 
+    if (!gjs_is_registered_native_module(js_context->context, NULL, "gi"))
+        gjs_register_native_module("gi", gjs_define_gi_stuff, 0);
+
+    /* For GjsDBus */
+    g_irepository_prepend_search_path(PKGLIBDIR);
+
     JS_EndRequest(js_context->context);
 
     g_static_mutex_lock (&contexts_lock);
